from typing import Union, Optional, Literal

from pydantic import Field

from pydantic_api.base import BaseModel
from .base import NotionPaginatedData, StartCursor, PageSize, SortObject
from ..objects import Page, Database


class SearchByTitleFilterObject(BaseModel):
    property: Literal["object"] = "object"
    value: Literal["page", "database"]


class SearchByTitleRequest(BaseModel):
    query: str = Field(
        ...,
        description="The text that the API compares page and database titles against.",
    )
    sort: Optional[SortObject] = Field(
        None,
        description='A set of criteria, direction and timestamp keys, that orders the results. The only supported timestamp value is "last_edited_time". Supported direction values are "ascending" and "descending". If sort is not provided, then the most recently edited results are returned first.',
    )
    filter: Optional[SearchByTitleFilterObject] = Field(
        None,
        description='A set of criteria, value and property keys, that limits the results to either only pages or only databases. Possible value values are "page" or "database". The only supported property value is "object".',
    )
    start_cursor: Optional[StartCursor] = None
    page_size: Optional[PageSize] = None


SearchByTitleResponse = Union[
    NotionPaginatedData[Union[Page, Database]],
    NotionPaginatedData[Page],
    NotionPaginatedData[Database],
]
"""Reference: https://developers.notion.com/reference/post-search"""


__all__ = [
    "SearchByTitleFilterObject",
    "SearchByTitleRequest",
    "SearchByTitleResponse",
]



from typing import Union, List, Dict, Optional

from uuid import UUID
from pydantic import Field, model_validator

from pydantic_api.base import BaseModel
from .base import NotionPaginatedData, FilterObject, SortObject, StartCursor, PageSize
from ..objects import (
    Page,
    Database,
    RichTextObject,
    PageParentObject,
    DatabaseProperty,
    IconObject,
    CoverObject,
)


class CreateDatabaseRequest(BaseModel):
    """
    Note:(quote from: https://developers.notion.com/reference/property-object#title)

    🚧 All databases require one, and only one, title property.

    The API throws errors if you send a request to Create a database without a title property, or if you attempt to Update a database to add or remove a title property.

    📘 Title database property vs. database title

    A title database property is a type of column in a database.

    A database title defines the title of the database and is found on the database object.

    Every database requires both a database title and a title database property.

    """

    parent: PageParentObject = Field(
        ...,
        description="A page parent",
    )
    title: List[RichTextObject] = Field(
        ...,
        description="Title of database as it appears in Notion. An array of rich text objects.",
    )
    properties: Dict[str, DatabaseProperty] = Field(
        ...,
        description="Property schema of database. The keys are the names of properties as they appear in Notion and the values are property schema objects.",
    )
    icon: Optional[IconObject] = Field(
        None,
        description="The icon of the database. Not listed in the documentation but shown in the official example.",
    )
    cover: Optional[CoverObject] = Field(
        None,
        description="The cover of the database. Not listed in the documentation but shown in the official example.",
    )

    @model_validator(mode="after")
    def validate_properties(self):
        # 1. properties must have one and only one title property
        title_property_count = 0
        for property in self.properties.values():
            if property.type == "title":
                title_property_count += 1
        if title_property_count == 0:
            raise ValueError(f"Database must have at least one title property")
        if title_property_count > 1:
            raise ValueError(
                f"Database must have only one title property, but found: {title_property_count}"
            )
        return self


CreateDatabaseResponse = Database
"""Reference: https://developers.notion.com/reference/create-a-database"""


class QueryDatabaseRequest(BaseModel):
    database_id: UUID = Field(..., description="Identifier for a Notion database.")
    filter: Optional[FilterObject] = Field(
        None,
        description="When supplied, limits which pages are returned based on the filter conditions.",
    )
    sorts: Optional[List[SortObject]] = Field(
        None,
        description="When supplied, orders the results based on the provided sort criteria.",
    )
    start_cursor: Optional[StartCursor] = None
    page_size: Optional[PageSize] = None


QueryDatabaseResponse = NotionPaginatedData[Union[Database, Page]]
"""Reference: https://developers.notion.com/reference/post-database-query"""


class RetrieveDatabaseRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/retrieve-a-database"""

    database_id: UUID = Field(..., description="An identifier for the Notion database.")


RetrieveDatabaseResponse = Database
"""Reference: https://developers.notion.com/reference/retrieve-a-database"""


class UpdateDatabaseRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/update-a-database

    Note: (quote from https://developers.notion.com/reference/update-a-database#Errors)

    > 🚧 The following database properties cannot be updated via the API:
    >
    >   - formula
    >   - select
    >   - status
    >   - Synced content
    >   - A multi_select database property's options values. An option can be removed, but not updated.

    """

    database_id: UUID = Field(
        ...,
        description="An identifier for the Notion database.",
    )
    title: List[RichTextObject] = Field(
        None,
        description="An array of rich text objects that represents the title of the database that is displayed in the Notion UI. If omitted, then the database title remains unchanged.",
    )
    description: List[RichTextObject] = Field(
        None,
        description="An array of rich text objects that represents the description of the database that is displayed in the Notion UI. If omitted, then the database description remains unchanged.",
    )
    properties: Dict[str, DatabaseProperty] = Field(
        None,
        description="The properties of a database to be changed in the request, in the form of a JSON object. If updating an existing property, then the keys are the names or IDs of the properties as they appear in Notion, and the values are property schema objects. If adding a new property, then the key is the name of the new database property and the value is a property schema object.",
    )


UpdateDatabaseResponse = Database
"""Reference: https://developers.notion.com/reference/update-a-database"""


__all__ = [
    "CreateDatabaseRequest",
    "CreateDatabaseResponse",
    "QueryDatabaseRequest",
    "QueryDatabaseResponse",
    "RetrieveDatabaseRequest",
    "RetrieveDatabaseResponse",
    "UpdateDatabaseRequest",
    "UpdateDatabaseResponse",
]



"""
Reference: https://developers.notion.com/reference/property-schema-object
"""

from typing import List, Optional, Literal, Union, Annotated

from uuid import UUID
from pydantic import Field, model_validator

from pydantic_api.base import BaseModel
from ..common import ColorLiteral
from .common import (
    SelectOption,
    RelationTypeLiteral,
    NumberFormatLiteral,
    RollupFunctionLiteral,
    DatabasePropertyTypeLiteral,
)


class EmptyConfig(BaseModel):
    pass


class BaseDatabaseProperty(BaseModel):
    id: Optional[str] = Field(
        None,
        description='An identifier for the property, usually a short string of random letters and symbols. Some automatically generated property types have special human-readable IDs. For example, all Title properties have an id of "title".',
    )
    name: Optional[str] = Field(
        None, description="The name of the property as it appears in Notion."
    )
    description: Optional[str] = Field(
        None, description="The description of the property as it appears in Notion."
    )
    type: DatabasePropertyTypeLiteral

    @classmethod
    def define(cls):
        return cls()


# Specific Database Property Schemas
class ButtonDatabaseProperty(BaseDatabaseProperty):
    type: Literal["button"] = "button"
    button: EmptyConfig = Field(default_factory=EmptyConfig)


class CheckboxDatabaseProperty(BaseDatabaseProperty):
    type: Literal["checkbox"] = "checkbox"
    checkbox: EmptyConfig = Field(default_factory=EmptyConfig)


class CreatedByDatabaseProperty(BaseDatabaseProperty):
    type: Literal["created_by"] = "created_by"
    created_by: EmptyConfig = Field(default_factory=EmptyConfig)


class CreatedTimeDatabaseProperty(BaseDatabaseProperty):
    type: Literal["created_time"] = "created_time"
    created_time: EmptyConfig = Field(default_factory=EmptyConfig)


class DateDatabaseProperty(BaseDatabaseProperty):
    type: Literal["date"] = "date"
    date: EmptyConfig = Field(default_factory=EmptyConfig)


class EmailDatabaseProperty(BaseDatabaseProperty):
    type: Literal["email"] = "email"
    email: EmptyConfig = Field(default_factory=EmptyConfig)


class FilesDatabaseProperty(BaseDatabaseProperty):
    """
    Note: The Notion API does not yet support uploading files to Notion.

    A files database property is rendered in the Notion UI as a column that has values that are either files uploaded directly to Notion or external links to files. The files type object is empty; there is no additional configuration.
    """

    type: Literal["files"] = "files"
    files: EmptyConfig = Field(default_factory=EmptyConfig)


# formula
class FormulaPropertyConfig(BaseModel):
    expression: str = Field(
        ...,
        description="The formula that is used to compute the values for this property. Refer to https://www.notion.so/help/formulas",
    )


class FormulaDatabaseProperty(BaseDatabaseProperty):
    type: Literal["formula"] = "formula"
    formula: FormulaPropertyConfig

    @classmethod
    def define(cls, expression: str):
        return cls(formula=FormulaPropertyConfig(expression=expression))


# last_edited_by
class LastEditedByDatabaseProperty(BaseDatabaseProperty):
    type: Literal["last_edited_by"] = "last_edited_by"
    last_edited_by: EmptyConfig = Field(default_factory=EmptyConfig)


# last_edited_time
class LastEditedTimeDatabaseProperty(BaseDatabaseProperty):
    type: Literal["last_edited_time"] = "last_edited_time"
    last_edited_time: EmptyConfig = Field(default_factory=EmptyConfig)


# multi_select
class MultiSelectPropertyConfig(BaseModel):
    options: List[SelectOption] = Field(default_factory=list)


class MultiSelectDatabaseProperty(BaseDatabaseProperty):
    type: Literal["multi_select"] = "multi_select"
    multi_select: MultiSelectPropertyConfig

    @classmethod
    def define(cls, options: List[SelectOption]):
        return cls(multi_select=MultiSelectPropertyConfig(options=options))


# number
class NumberPropertyConfig(BaseModel):
    format: Optional[NumberFormatLiteral] = Field(None)


class NumberDatabaseProperty(BaseDatabaseProperty):
    type: Literal["number"] = "number"
    number: NumberPropertyConfig = Field(default_factory=NumberPropertyConfig)

    @classmethod
    def define(cls, format: Optional[NumberFormatLiteral] = None):
        return cls(number=NumberPropertyConfig(format=format))


# people
class PeopleDatabaseProperty(BaseDatabaseProperty):
    type: Literal["people"] = "people"
    people: EmptyConfig = Field(default_factory=EmptyConfig)


# phone_number
class PhoneNumberDatabaseProperty(BaseDatabaseProperty):
    type: Literal["phone_number"] = "phone_number"
    phone_number: EmptyConfig = Field(default_factory=EmptyConfig)


# relation
class SinglePropertyData(BaseModel):
    pass


class SinglePropertyRelationData(BaseModel):
    type: Literal["single_property"] = "single_property"
    database_id: UUID = Field(
        ...,
        description="The database that the relation property refers to. The corresponding linked page values must belong to the database in order to be valid.",
    )
    single_property: SinglePropertyData = Field(
        default_factory=SinglePropertyData,
        description="The configuration for a single property relation.",
    )


class SinglePropertyRelationDatabaseProperty(BaseModel):
    type: Literal["relation"] = "relation"
    relation: SinglePropertyRelationData


class DualPropertyData(BaseModel):
    pass


class DualPropertyData(BaseModel):
    type: Literal["dual_property"] = "dual_property"
    database_id: UUID = Field(
        ...,
        description="The database that the relation property refers to. The corresponding linked page values must belong to the database in order to be valid.",
    )
    dual_property: DualPropertyData = Field(
        default_factory=DualPropertyData,
        description="The configuration for a dual property relation.",
    )


class DualPropertyRelationDatabaseProperty(BaseModel):
    type: Literal["relation"] = "relation"
    relation: DualPropertyData


RelationDatabaseProperty = Union[
    SinglePropertyRelationDatabaseProperty, DualPropertyRelationDatabaseProperty
]


# rich_text
class RichTextDatabaseProperty(BaseDatabaseProperty):
    type: Literal["rich_text"] = "rich_text"
    rich_text: EmptyConfig = Field(default_factory=EmptyConfig)


# rollup
class RollupPropertyConfig(BaseModel):
    relation_property_name: Optional[str] = None
    relation_property_id: Optional[str] = None
    rollup_property_name: Optional[str] = None
    rollup_property_id: Optional[str] = None
    function: RollupFunctionLiteral

    @model_validator(mode="after")
    def ensure_either_name_or_id_is_provided(self):
        if self.relation_property_name is None and self.relation_property_id is None:
            raise ValueError(
                "Either relation_property_name or relation_property_id is required."
            )
        if self.rollup_property_name is None and self.rollup_property_id is None:
            raise ValueError(
                "Either rollup_property_name or rollup_property_id is required."
            )
        return self


class RollupDatabaseProperty(BaseDatabaseProperty):
    type: Literal["rollup"] = "rollup"
    rollup: RollupPropertyConfig

    @classmethod
    def define(
        cls,
        function: RollupFunctionLiteral,
        relation_property_name: Optional[str] = None,
        relation_property_id: Optional[str] = None,
        rollup_property_name: Optional[str] = None,
        rollup_property_id: Optional[str] = None,
    ):
        return cls(
            rollup=RollupPropertyConfig(
                function=function,
                relation_property_name=relation_property_name,
                relation_property_id=relation_property_id,
                rollup_property_name=rollup_property_name,
                rollup_property_id=rollup_property_id,
            )
        )


# select
class SelectPropertyConfig(BaseModel):
    options: List[SelectOption] = Field(default_factory=list)


class SelectDatabaseProperty(BaseDatabaseProperty):
    type: Literal["select"] = "select"
    select: SelectPropertyConfig = Field(default_factory=SelectPropertyConfig)

    @classmethod
    def define(cls, options: List[SelectOption]):
        return cls(select=SelectPropertyConfig(options=options))


# status
class StatusOption(BaseModel):
    color: Optional[ColorLiteral] = Field(None)
    id: Optional[str] = Field(None)
    name: Optional[str] = Field(None)


class StatusGroup(BaseModel):
    color: Optional[ColorLiteral] = Field(None)
    id: Optional[str] = Field(None)
    name: Optional[str] = Field(None)
    option_ids: Optional[List[str]] = Field(
        None,
        description="A sorted list of ids of all of the options that belong to a group.",
    )


class StatusPropertyConfig(BaseModel):
    options: List[StatusOption] = Field(default_factory=list)
    group: Optional[List[StatusGroup]] = Field(default_factory=list)


class StatusDatabaseProperty(BaseDatabaseProperty):
    """
    Note: It is not possible to update a status database property's name or options values via the API.
    """

    type: Literal["status"] = "status"
    status: StatusPropertyConfig


# title
class TitleDatabaseProperty(BaseDatabaseProperty):
    type: Literal["title"] = "title"
    title: EmptyConfig = Field(default_factory=EmptyConfig)


# url
class URLDatabaseProperty(BaseDatabaseProperty):
    type: Literal["url"] = "url"
    url: EmptyConfig = Field(default_factory=EmptyConfig)


# unique id
class UniqueIdPropertyConfig(EmptyConfig):
    """
    The configuration for a unique ID property.
    """

    prefix: str | None = None


class UniqueIdDatabaseProperty(BaseDatabaseProperty):
    type: Literal["unique_id"] = "unique_id"
    unique_id: UniqueIdPropertyConfig

    @classmethod
    def define(cls, prefix: str | None = None):
        return cls(unique_id=UniqueIdPropertyConfig(prefix=prefix))


# Union for all Database Schema Properties
DatabaseProperty = Union[
    ButtonDatabaseProperty,
    CheckboxDatabaseProperty,
    CreatedByDatabaseProperty,
    CreatedTimeDatabaseProperty,
    DateDatabaseProperty,
    EmailDatabaseProperty,
    FilesDatabaseProperty,
    FormulaDatabaseProperty,
    LastEditedByDatabaseProperty,
    LastEditedTimeDatabaseProperty,
    MultiSelectDatabaseProperty,
    NumberDatabaseProperty,
    PeopleDatabaseProperty,
    PhoneNumberDatabaseProperty,
    RelationDatabaseProperty,
    RichTextDatabaseProperty,
    RollupDatabaseProperty,
    SelectDatabaseProperty,
    StatusDatabaseProperty,
    TitleDatabaseProperty,
    URLDatabaseProperty,
    UniqueIdDatabaseProperty,
]


__all__ = [
    "ButtonDatabaseProperty",
    "CheckboxDatabaseProperty",
    "CreatedByDatabaseProperty",
    "CreatedTimeDatabaseProperty",
    "DateDatabaseProperty",
    "EmailDatabaseProperty",
    "FilesDatabaseProperty",
    "FormulaDatabaseProperty",
    "LastEditedByDatabaseProperty",
    "LastEditedTimeDatabaseProperty",
    "MultiSelectDatabaseProperty",
    "NumberDatabaseProperty",
    "PeopleDatabaseProperty",
    "PhoneNumberDatabaseProperty",
    "RelationDatabaseProperty",
    "RichTextDatabaseProperty",
    "RollupDatabaseProperty",
    "SelectDatabaseProperty",
    "StatusDatabaseProperty",
    "TitleDatabaseProperty",
    "URLDatabaseProperty",
    "UniqueIdDatabaseProperty",
    # Union Type
    "DatabaseProperty",
]



from typing import List, Optional

from uuid import UUID
from pydantic import BaseModel, Field, model_validator

from ..objects import CommentObject, RichTextObject, PageParentObject
from .base import NotionPaginatedData, StartCursor, PageSize


class CreateCommentRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/create-a-comment"""

    parent: Optional[PageParentObject] = Field(
        None,
        description="A page parent. Either this or a discussion_id is required (not both)",
    )
    discussion_id: Optional[UUID] = Field(
        None,
        description="A UUID identifier for a discussion thread. Either this or a parent object is required (not both)",
    )
    rich_text: List[RichTextObject] = Field(..., description="A rich text object")

    @model_validator(mode="after")
    def either_parent_or_discussionId_must_be_provided(self):
        if not self.parent and not self.discussion_id:
            raise ValueError("Either parent or discussion_id must be provided")
        return self


CreateCommentResponse = CommentObject
"""Reference: https://developers.notion.com/reference/create-a-comment"""


class RetrieveCommentsRequest(BaseModel):
    """Reference: https://developers.notion.com/reference/retrieve-a-comment"""

    block_id: UUID = Field(
        ...,
        description="Identifier for a Notion block or page, a uuidv4 string. Reference: https://developers.notion.com/reference/block#keys",
    )
    start_cursor: Optional[StartCursor] = None
    page_size: Optional[PageSize] = None


RetrieveCommentsResponse = NotionPaginatedData[CommentObject]
"""Reference: https://developers.notion.com/reference/retrieve-a-comment"""


__all__ = [
    "CreateCommentRequest",
    "CreateCommentResponse",
    "RetrieveCommentsRequest",
    "RetrieveCommentsResponse",
]



from typing import Literal

from pydantic_api.base import BaseModel
from .base import BaseRichTextObject


class EquationObject(BaseModel):
    """Equation object type."""

    expression: str


class EquationRichTextObject(BaseRichTextObject):
    """Rich Text type: Equation."""

    type: Literal["equation"] = "equation"
    equation: EquationObject

    @classmethod
    def new(cls, expression: str):
        return cls(equation=EquationObject(expression=expression))


__all__ = [
    "EquationObject",
    "EquationRichTextObject",
]



"""
Reference: https://developers.notion.com/reference/emoji-object
"""

from typing import Literal

import emoji
from pydantic import field_validator, Field

from pydantic_api.base import BaseModel


class EmojiObject(BaseModel):
    """Reference: https://developers.notion.com/reference/emoji-object"""

    type: Literal["emoji"] = "emoji"
    emoji: str = Field(..., description="The emoji character.", examples=["😻"])

    @field_validator("emoji")
    @classmethod
    def ensure_valid_emoji_character(cls, v: str):
        # if len(v) > 1:
        # raise ValueError("Emoji must be a single character.")
        if not emoji.is_emoji(v):
            raise ValueError("Invalid emoji character: {v}")
        return v


__all__ = [
    "EmojiObject",
]



from datetime import datetime
from typing import Literal, Union, Annotated, Optional

from uuid import UUID
from pydantic import Field

from pydantic_api.base import BaseModel
from .base import BaseRichTextObject

MentionTypeLiteral = Literal[
    "database",
    "date",
    "link_preview",
    "page",
    "template_mention",
    "user",
]


class _BaseMentionObject(BaseModel):
    """Base model for Mention Object."""

    type: MentionTypeLiteral


# 1. Database mention
class DatabaseMentionBody(BaseModel):
    id: UUID = Field(..., description=f"The id of the mentioned database")


class DatabaseMentionObject(_BaseMentionObject):
    """Mention type: Database."""

    type: Literal["database"] = "database"
    database: DatabaseMentionBody


# 2. Date mention
class DateMentionBody(BaseModel):
    start: datetime
    end: Optional[datetime] = None
    time_zone: Optional[str] = Field(
        None,
        description="An enum value from IANA database based on Moment.js",
    )


class DateMentionObject(_BaseMentionObject):
    """Mention type: Date."""

    type: Literal["date"] = "date"
    date: DateMentionBody


# 3. LinkPreview mention
class LinkPreviewMentionBody(BaseModel):
    url: str


class LinkPreviewMentionObject(_BaseMentionObject):
    """Mention type: Link Preview."""

    type: Literal["link_preview"] = "link_preview"
    link_preview: LinkPreviewMentionBody


# 4. Page mention
class PageMentionBody(BaseModel):
    id: UUID = Field(..., description=f"The id of the mentioned page")


class PageMentionObject(_BaseMentionObject):
    """Mention type: Page."""

    type: Literal["page"] = "page"
    page: PageMentionBody


# 5. Tempalte mention
TemplateMentionTypeLiteral = Literal["template_mention_date", "template_mention_user"]


TemplateMentionDateValueLiteral = Literal["today", "now"]


TemplateMentionUserValueLiteral = Literal["me"]


class _BaseTemplateMentionBody(BaseModel):
    type: TemplateMentionTypeLiteral


class _TemplateMentionDate(_BaseTemplateMentionBody):
    type: Literal["template_mention_date"] = "template_mention_date"

    template_mention_date: TemplateMentionDateValueLiteral


class _TemplateMentionUser(_BaseTemplateMentionBody):
    type: Literal["template_mention_user"] = "template_mention_user"

    template_mention_user: TemplateMentionUserValueLiteral


TemplateMentionBody = Annotated[
    Union[_TemplateMentionDate, _TemplateMentionUser], Field(discriminator="type")
]


class TemplateMentionObject(_BaseMentionObject):
    """Mention type: Template Mention."""

    type: Literal["template_mention"] = "template_mention"
    template_mention: TemplateMentionBody


# 6. User mention
class UserMentionBody(BaseModel):
    object: Literal["user"] = "user"
    id: UUID = Field(..., description="The id of the mentioned user")


class UserMentionObject(_BaseMentionObject):
    """Mention type: User."""

    type: Literal["user"] = "user"
    user: UserMentionBody


MentionObject = Annotated[
    Union[
        DatabaseMentionObject,
        DateMentionObject,
        LinkPreviewMentionObject,
        PageMentionObject,
        TemplateMentionObject,
        UserMentionObject,
    ],
    Field(discriminator="type"),
]


class MentionRichTextObject(BaseRichTextObject):
    """Rich Text type: Mention."""

    type: Literal["mention"] = "mention"
    mention: MentionObject


__all__ = [
    "MentionTypeLiteral",
    "DatabaseMentionBody",
    "DatabaseMentionObject",
    "DateMentionBody",
    "DateMentionObject",
    "LinkPreviewMentionBody",
    "LinkPreviewMentionObject",
    "PageMentionBody",
    "PageMentionObject",
    "TemplateMentionBody",
    "TemplateMentionObject",
    "TemplateMentionTypeLiteral",
    "TemplateMentionDateValueLiteral",
    "TemplateMentionUserValueLiteral",
    "UserMentionBody",
    "UserMentionObject",
    "MentionObject",
    "MentionRichTextObject",
]



from typing import Union, Literal

from .emoji import EmojiObject
from .file import ExternalFileObject


IconObject = Union[ExternalFileObject, EmojiObject]


class IconObjectFactory:
    @classmethod
    def from_external_file(cls, url: str, name: str | None = None):
        return ExternalFileObject.new(
            url=url,
            name=name,
        )

    @classmethod
    def from_emoji(cls, emoji: str):
        """Create a new IconObject with emoji.

        Args:
            emoji: The emoji to use.
        """
        return EmojiObject(emoji=emoji)


CoverObject = ExternalFileObject


class CoverObjectFactory:
    @classmethod
    def from_external_file(cls, url: str):
        return ExternalFileObject.from_url(url=url)


ColorLiteral = Literal[
    "default",
    "gray",
    "gray_background",
    "brown",
    "brown_background",
    "orange",
    "orange_background",
    "yellow",
    "yellow_background",
    "green",
    "green_background",
    "blue",
    "blue_background",
    "purple",
    "purple_background",
    "pink",
    "pink_background",
    "red",
    "red_background",
]

CodeLanguageLiteral = Literal[
    "abap",
    "arduino",
    "bash",
    "basic",
    "c",
    "clojure",
    "coffeescript",
    "c++",
    "c#",
    "css",
    "dart",
    "diff",
    "docker",
    "elixir",
    "elm",
    "erlang",
    "flow",
    "fortran",
    "f#",
    "gherkin",
    "glsl",
    "go",
    "graphql",
    "groovy",
    "haskell",
    "html",
    "java",
    "javascript",
    "json",
    "julia",
    "kotlin",
    "latex",
    "less",
    "lisp",
    "livescript",
    "lua",
    "makefile",
    "markdown",
    "markup",
    "matlab",
    "mermaid",
    "nix",
    "objective-c",
    "ocaml",
    "pascal",
    "perl",
    "php",
    "plain text",
    "powershell",
    "prolog",
    "protobuf",
    "python",
    "r",
    "reason",
    "ruby",
    "rust",
    "sass",
    "scala",
    "scheme",
    "scss",
    "shell",
    "sql",
    "swift",
    "typescript",
    "vb.net",
    "verilog",
    "vhdl",
    "visual basic",
    "webassembly",
    "xml",
    "yaml",
    "java/c/c++/c#",
]
"""Reference: https://developers.notion.com/reference/block#code"""


__all__ = [
    "IconObject",
    "IconObjectFactory",
    "CoverObject",
    "CoverObjectFactory",
    "ColorLiteral",
    "CodeLanguageLiteral",
]



from typing import Literal, Optional

from pydantic import Field

from pydantic_api.base import BaseModel

from pydantic_api.notion.models.objects.common import ColorLiteral


class TextAnnotations(BaseModel):
    """Text style annotations for rich text."""

    bold: bool
    italic: bool
    strikethrough: bool
    underline: bool
    code: bool
    color: ColorLiteral


RichTextTypeLiteral = Literal["text", "mention", "equation"]


class BaseRichTextObject(BaseModel):
    """Base model for Rich Text."""

    type: RichTextTypeLiteral
    annotations: Optional[TextAnnotations] = Field(
        None, description="Formatting style for the text"
    )
    plain_text: Optional[str] = Field(None)
    href: Optional[str] = Field(None, description="Hyperlink for the text")


__all__ = [
    "TextAnnotations",
    "RichTextTypeLiteral",
    "BaseRichTextObject",
]



from typing import Literal, Optional

from pydantic import Field, model_validator

from pydantic_api.base import BaseModel
from ..common import ColorLiteral


class SelectOption(BaseModel):
    """
    Selection Option Object.

    Reference:

    - https://developers.notion.com/reference/property-schema#select
    - https://developers.notion.com/reference/page-property-values#select
    """

    color: Optional[ColorLiteral] = Field(None)
    id: Optional[str] = None
    name: Optional[str] = None

    @model_validator(mode="after")
    def either_id_or_name_is_provided(self):
        if self.id is None and self.name is None:
            raise ValueError(f"Either id or name must be provided")
        return self

    @classmethod
    def new(cls, name: str, color: ColorLiteral | None = None):
        """Constructor used when creating a new SelectOption object."""
        return cls(name=name, color=color)

    @classmethod
    def refer(cls, name: str):
        """Constructor used when referring to an existing SelectOption object."""
        return cls(name=name)


class StatusOption(BaseModel):
    color: Optional[ColorLiteral] = Field(None)
    id: Optional[str] = None
    name: Optional[str] = None

    @model_validator(mode="after")
    def either_id_or_name_should_be_provided(self):
        if self.id is None and self.name is None:
            raise ValueError(f"Either name or id must be provided")
        return self

    @classmethod
    def new(cls, name: str, color: ColorLiteral | None = None):
        """Constructor used when creating a new StatusObject."""
        return cls(name=name, color=color)

    @classmethod
    def refer(cls, name: str):
        """Constructor used when referring to an existing StatusObject."""
        return cls(name=name)


DatabasePropertyTypeLiteral = Literal[
    "button",
    "checkbox",
    "created_by",
    "created_time",
    "date",
    "email",
    "files",
    "formula",
    "last_edited_by",
    "last_edited_time",
    "multi_select",
    "number",
    "people",
    "phone_number",
    "relation",
    "rich_text",
    "rollup",
    "select",
    "status",
    "title",
    "url",
    "unique_id",
]
"""
Database Property Type Enum(typed as Literal in this implementation).

Reference: https://developers.notion.com/reference/property-object
"""

PagePropertyTypeLiteral = Literal[
    "button",
    "checkbox",
    "created_by",
    "created_time",
    "date",
    "email",
    "files",
    "formula",
    "last_edited_by",
    "last_edited_time",
    "multi_select",
    "number",
    "people",
    "phone_number",
    "relation",
    "rich_text",
    "rollup",
    "select",
    "status",
    "title",
    "url",
    "unique_id",
    "verification",
]
"""
Page Property Type Enum(typed as Literal in this implementation).

Reference: https://developers.notion.com/reference/page-property-values
"""


FormulaValueTypeLiteral = Literal["boolean", "date", "number", "string"]
"""Reference: https://developers.notion.com/reference/page-property-values#formula"""

RelationTypeLiteral = Literal["single_property", "dual_property"]
"""Reference: https://developers.notion.com/reference/page-property-values#relation"""

RollupFunctionLiteral = Literal[
    "count_all",
    "count_values",
    "count_unique_values",
    "count_empty",
    "count_not_empty",
    "percent_empty",
    "percent_not_empty",
    "sum",
    "average",
    "median",
    "min",
    "max",
    "range",
    "show_original",
]
"""Reference: https://developers.notion.com/reference/page-property-values#rollup"""

RollupTypeLiteral = Literal["array", "date", "incomplete", "number", "unsupported"]
"""Reference: https://developers.notion.com/reference/page-property-values#rollup"""

NumberFormatLiteral = Literal[
    "number",
    "number_with_commas",
    "percent",
    "dollar",
    "canadian_dollar",
    "euro",
    "pound",
    "yen",
    "ruble",
    "rupee",
    "won",
    "yuan",
    "real",
    "lira",
    "rupiah",
    "franc",
    "hong_kong_dollar",
    "new_zealand_dollar",
    "krona",
    "norwegian_krone",
    "mexican_peso",
    "rand",
    "new_taiwan_dollar",
    "danish_krone",
    "zloty",
    "baht",
    "forint",
    "koruna",
    "shekel",
    "chilean_peso",
    "philippine_peso",
    "dirham",
    "colombian_peso",
    "riyal",
    "ringgit",
    "leu",
    "argentine_peso",
    "uruguayan_peso",
    "singapore_dollar",
]
"""Reference: https://developers.notion.com/reference/page-property-values#number"""

VerificationStateLiteral = Literal["verified", "unvirified"]
"""Reference: https://developers.notion.com/reference/page-property-values#verification"""

__all__ = [
    "SelectOption",
    "StatusOption",
    "FormulaValueTypeLiteral",
    "RelationTypeLiteral",
    "RollupFunctionLiteral",
    "RollupTypeLiteral",
    "NumberFormatLiteral",
    "VerificationStateLiteral",
    "PagePropertyTypeLiteral",
    "DatabasePropertyTypeLiteral",
]



"""
Reference: https://developers.notion.com/reference/rich-text
"""

from .base import TextAnnotations, RichTextTypeLiteral
from .text import TextObject, TextRichTextObject
from .equation import EquationObject, EquationRichTextObject
from .mention import (
    MentionTypeLiteral,
    DatabaseMentionBody,
    DatabaseMentionObject,
    DateMentionBody,
    DateMentionObject,
    LinkPreviewMentionBody,
    LinkPreviewMentionObject,
    PageMentionBody,
    PageMentionObject,
    TemplateMentionBody,
    TemplateMentionObject,
    TemplateMentionTypeLiteral,
    TemplateMentionDateValueLiteral,
    TemplateMentionUserValueLiteral,
    UserMentionBody,
    UserMentionObject,
    MentionObject,
    MentionRichTextObject,
)
from .rich_text import RichTextObject, RichTextObjectFactory


__all__ = [
    # Base
    "TextAnnotations",
    "RichTextTypeLiteral",
    # Text
    "TextObject",
    "TextRichTextObject",
    # Equation
    "EquationObject",
    "EquationRichTextObject",
    # Mention
    "MentionTypeLiteral",
    "DatabaseMentionBody",
    "DatabaseMentionObject",
    "DateMentionBody",
    "DateMentionObject",
    "LinkPreviewMentionBody",
    "LinkPreviewMentionObject",
    "PageMentionBody",
    "PageMentionObject",
    "TemplateMentionBody",
    "TemplateMentionObject",
    "TemplateMentionTypeLiteral",
    "TemplateMentionDateValueLiteral",
    "TemplateMentionUserValueLiteral",
    "UserMentionBody",
    "UserMentionObject",
    "MentionObject",
    "MentionRichTextObject",
    # Rich Text
    "RichTextObject",
    "RichTextObjectFactory",
]
